<?php

class GelProximityApiS2s {
	protected $apiEndpoint;
	protected $apiKey;
	protected $merchantCode;
	protected $accessToken;
	protected $locale;

	public function __construct( $api_endpoint, $api_key, $merchantCode, $locale = '' ) {
		$this->apiEndpoint  = $api_endpoint;
		$this->apiKey       = $api_key;
		$this->merchantCode = $merchantCode;
		$this->accessToken  = '';
		if ( empty( $locale ) ) {
			$this->locale = GelProximityConfiguration::GELPRX_CONFIG_DEFAULT_LOCALE;
		}
	}

	// Method: POST, PUT, GET etc
	// Data: array("param" => "value") ==> index.php?param=value
	protected function callAPI( $method, $url, $body = false ) {
		GelProximityUtility::doLog( $body, 'callAPI body' );

		$certificate = plugin_dir_path( __DIR__ ) . 'cacert.pem';

		$args = array(
			'body'            => $body,
			'timeout'         => '5',
			'redirection'     => '5',
			'httpversion'     => '1.0',
			'blocking'        => true,
			'headers'         => array(),
			'cookies'         => array(),
			'sslverify'       => true,
			'sslcertificates' => $certificate,
		);

		switch ( $method ) {
			case "POST":
				$response = wp_remote_post( $url, $args );
				break;
			case "PUT":
				$response = wp_remote_request( $url, $args );
				break;
			default:
				if ( $body ) {
					$url = sprintf( "%s?%s", $url, http_build_query( $body ) );
				}
				$response = wp_remote_get( $url );
		}

		GelProximityUtility::doLog( $response, 'callAPI $response' );

		return $response;
	}

	public function auth() {
		$url  = $this->apiEndpoint . '/api/merchant/autenticate';
		$data = array(
			'apiKey'       => $this->apiKey,
			'merchantCode' => $this->merchantCode,
			'locale'       => $this->locale,
		);
		try {
			$response = $this->callAPI( 'POST', $url, GelProximityUtility::doSerialize( $data ) );
			GelProximityUtility::doLog( $response, 'API auth' );
			$response_unserialized = GelProximityUtility::doUnserialize( $response, 'json', array( 'assoc' => true ) );

			// TODO: set accessToken
			return $response_unserialized;
		} catch ( Exception $e ) {
			return false;
		}
	}

	// reference: riferimento all’ordine e-commerce.
	//            Deve essere lo stesso valore utilizzato nell’instanziamento del componente SDK
	// orderDataJson: valore stringa contenente un json con il dettaglio dell’ordine
	// isReturnOrder:  campo da indicare nel caso si tratti un ordine di reso (default false)
	public function buildOrderDataJson( &$order, &$payload ) // protected
	{
		if ( ! is_array( $payload ) ) {
			$payload = GelProximityUtility::doUnserialize( $payload, 'json', array( 'assoc' => true ) );
		}

		$orderArr = array(
			'orderNumber'   => $order->get_id(),
			// numero d’ordine, alfanumerico, obbligatorio
			'orderDate'     => date( "Ymd", strtotime( $order->get_date_created() ) ),
			// data dell’ordine, alfanumerico nel formato yyyyMMdd, obbligatorio
			'description'   => $order->get_order_key(),
			// descrizione dell’ordine, alfanumerico, obbligatorio
			'customer'      => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
			// denominazione cliente, alfanumerico, obbligatorio
			'customerEmail' => $order->get_billing_email(),
			// email cliente, alfanumerico, obbligatorio
		);

		$rows                    = array();
		$products                = $order->get_items();
		$woocommerce_weight_unit = trim( get_option( 'woocommerce_weight_unit' ) );
		$order_total_weight      = 0;
		foreach ( $products as $item ) {
			$product = $item->get_product();

			// + T17734 Singolo prodotto
			$grossWeight = '';
			$netWeight   = '';
			$height      = '';
			$width       = '';
			$depth       = '';

			// TODO: conversion if $woocommerce_weight_unit !== kg
			if ( $woocommerce_weight_unit === 'kg' && ! $product->is_virtual() ) {
				$grossWeight        += (float) $product->get_weight();
				$netWeight          = $grossWeight;
				$height             = (float) $product->get_height();
				$width              = (float) $product->get_width();
				$depth              = (float) $product->get_length();
				$order_total_weight += (float) $grossWeight * $item['qty'];
			}
			// - T17734 Singolo prodotto


			$rows[] = array(
				'description' => $item->get_name(),
				'productCode' => $product->get_id(), // get_sku()
				// $product['product_reference'],
				'grossWeight' => $grossWeight, // Kg
				'netWeight'   => $netWeight, // Kg
				'height'      => $height, // cm
				'width'       => $width, // cm
				'depth'       => $depth, // cm
			);
		}

		// + T17734 Ordine
		$grossWeight = empty( $order_total_weight ) ? '' : $order_total_weight;
		$netWeight   = $grossWeight;
		$height      = '';
		$width       = '';
		$depth       = '';
		// - T17734 Ordine

		// Facoltativi
		$orderArrExtra = array(
			'customerPhone'           => $order->get_billing_phone(),
			'codePickingPoint'        => $payload['pickupPointId'],
			// Numero progressivo dei pacchi spediti dall'ecommerce nell'anno corrente
			'orderCountInCurrentYear' => GelProximityOrderHistory::getOrderCountInCurrentYear(),
			// Valore ordine: totale costo complessivo dei prodotti in euro IVA esclusa
			'goodsValue'              => (string) $order->get_subtotal(),
			'currencyGoodsValue'      => strtoupper( $order->get_currency() ), // valuta
			'piecesCount'             => $order->get_item_count(),
			'grossWeight'             => $grossWeight, // Kg
			'netWeight'               => $netWeight, // Kg
			'height'                  => $height, // cm
			'width'                   => $width, // cm
			'depth'                   => $depth, // cm
			'rows'                    => $rows,
		);

		return GelProximityUtility::doSerialize( array_merge( $orderArr, $orderArrExtra ) );
	}

	// Conferma ordine
	// success: flag boolean che indica se la conferma è riuscita
	// message: messaggio, utile in caso di success false
	public function confirmOrder( $reference, $order, $payload ) {
		$url  = $this->apiEndpoint . '/api/confirmOrder';
		$data = array(
			'apiKey'        => $this->apiKey,
			'merchantCode'  => $this->merchantCode,
			'reference'     => $reference, // id_cart
			'orderDataJson' => $this->buildOrderDataJson( $order, $payload ),
			'isReturnOrder' => false,
		);
		try {
			$response = $this->callAPI( 'POST', $url, $data );

			if ( is_wp_error( $response ) ) {
				$error_message = $response->get_error_message();
				GelProximityUtility::doLog( $error_message, 'API confirmOrder Error' );

				return false;
			}

			if ( isset( $response['body'] ) && ! empty( $response['body'] ) ) {
				GelProximityUtility::doLog( $response['body'], 'API confirmOrder' );

				return GelProximityUtility::doUnserialize( $response['body'], 'json', array( 'assoc' => true ) );
			} else {
				return false;
			}
		} catch ( Exception $e ) {
			return false;
		}
	}
}
